use rusqlite::{Connection, Result};
use std::time::SystemTime;

#[allow(dead_code)]
pub struct Suspicious {
    ip_addr: String,
    banned: u8,
    create_time: u64,
}

pub fn init_db(path: &str) -> Result<Connection> {
    let conn = Connection::open(path)?;
    conn.execute(
        "CREATE TABLE IF NOT EXISTS suspicious (
            ip_addr VARCHAR(128) PRIMARY KEY,
            banned INTEGER NOT NULL,
            create_time INTEGER NOT NULL
        )", [])?;

    Ok(conn)
}

#[allow(dead_code)]
pub fn get_all_suspicious(conn: &Connection) -> Result<Vec<Suspicious>, rusqlite::Error> {
    let mut stmt = conn.prepare("SELECT * FROM suspicious")?;
    let rows = stmt.query_map([], |row| {
        Ok(Suspicious {
            ip_addr: row.get(0)?,
            banned: row.get(1)?,
            create_time: row.get(2)?,
        })
    })?;

    Ok(rows.collect::<Result<Vec<Suspicious>, _>>()?)
}

pub fn get_suspicious_by_ip_addr(conn: &Connection, ip_addr: &str) -> Result<Suspicious, rusqlite::Error> {
    let mut stmt = conn.prepare("SELECT * FROM suspicious WHERE ip_addr = ?")?;
    let row = stmt.query_row([ip_addr], |row| {
        Ok(Suspicious {
            ip_addr: row.get(0)?,
            banned: row.get(1)?,
            create_time: row.get(2)?,
        })
    })?;
    Ok(row)
}

pub fn add_suspicious(conn: &Connection, mut ip_addr: &str) -> Result<(), rusqlite::Error> {

    //if ip_addr has ":" in it means it has port, we insert only IP addr
    if ip_addr.contains(":") {
        ip_addr = ip_addr.split(":").collect::<Vec<&str>>()[0];
    }

    let epoch = SystemTime::now()
        .duration_since(SystemTime::UNIX_EPOCH)
        .unwrap()
        .as_secs()
        .to_string();

    let mut stmt = conn.prepare("INSERT OR IGNORE INTO suspicious (ip_addr, banned, create_time) VALUES (?, ?, ?)")?;
    stmt.execute([ip_addr, "0", epoch.as_str()])?;
    Ok(())
}

#[allow(dead_code)]
pub fn delete_suspicious(conn: &Connection, ip_addr: &str) -> Result<(), rusqlite::Error> {
    let mut stmt = conn.prepare("DELETE FROM suspicious WHERE ip_addr = ?")?;
    stmt.execute([ip_addr])?;
    Ok(())
}
