// Mandatory: To, From, CSeq, Call-ID, Max-Forwards, and Via;
//
#[derive(Debug)]
pub struct Options {
    pub command: String,             // OPTIONS sip:100@127.0.0.1 SIP/2.0
    pub via: String,                 // SIP/2.0/UDP 127.0.1.1:5061;branch=z9hG4bK-487618168;rport
    pub max_forwards: i32,           // 70
    pub to: String,                  // "sipvicious"<sip:100@1.1.1.1>
    pub from: String, // "sipvicious"<sip:100@1.1.1.1>;tag=37663030303030313133633401393537303038303638
    pub user_agent: Option<String>, // friendly-scanner
    pub call_id: Option<String>, // Call-ID: 447615548427934163033914
    pub contact: Option<String>, // sip:100@127.0.1.1:5061
    pub cseq: String, // 1 OPTIONS
    pub accept: Option<String>, // application/sdp
    pub content_length: Option<i32>, // 0
}

impl Options {
    pub fn new() -> Options {
        Options {
            command: "OPTIONS".to_string(),
            via: String::new(),
            max_forwards: 0,
            to: String::new(),
            from: String::new(),
            user_agent: Some(String::new()),
            call_id: Some(String::new()),
            contact: Some(String::new()),
            cseq: String::new(),
            accept: Some(String::new()),
            content_length: Some(0),
        }
    }

    pub fn parse(msg: &String) -> Options {
        let mut packet = Options::new();
        for line in msg.lines() {
            if line.len() > 0 {
                // ata os primeiros ":" é a cabeceira e logo o contido
                let header = line.chars()
                   .take_while(|&ch| ch != ':')
                   .collect::<String>();
                let mut content = line.chars()
                   .skip(header.len()+1)
                   .collect::<String>();
                content = content.trim().to_string();

                match &header[..] {
                    "OPTIONS sip" => packet.command = format!("{}{}", header, content),
                    "Via" => packet.via = content.to_string(),
                    "Max-Forwards" => packet.max_forwards = content.parse::<i32>().unwrap(),
                    "To" => packet.to = content.to_string(),
                    "From" => packet.from = content.to_string(),
                    "User-Agent" => packet.user_agent = Some(content.to_string()),
                    "Call-ID" => packet.call_id = Some(content.to_string()),
                    "Contact" => packet.contact = Some(content.to_string()),
                    "CSeq" => packet.cseq = content.to_string(),
                    "Accept" => packet.accept = Some(content.to_string()),
                    "Content-Length" => {
                        packet.content_length = Some(content.parse::<i32>().unwrap())
                    }
                    _ => println!("---> {:?} no contemplada!", header),
                }
            }
        }
        // println!("{:?}",packet);
        packet
    }
}
